function histscal_disp(s,smooth,bin,opt)
%HISTSCAL_DISP  Displays distributions of scalar fields.
%   HISTSCAL_DISP(S) displays the distributions of the scalar field(s) S.
%
%   HISTSCAL_DISP(S,SMOOTH) computes the velocity distributions averaged
%   over SMOOTH consecutive fields. Use SMOOTH=1 for no smoothing, or
%   SMOOTH=0 to smooth over all the fields (by default).
%
%   HISTSCAL_DISP(S,SMOOTH,BIN) computes the histograms among the bins
%   specified by the vector BIN. If this argument is not present, the
%   appropriate range of bins is estimated from the rms of the first
%   field.
%
%   HISTSCAL_DISP(...,OPT), where OPT is a string that may contain one or
%   more of the following letters (all by default):
%        'n': normalize the histogram (pdf)
%        'g': also displays a gaussian fit
%        'l': log-coordinate for the y axis
%
%   Example:
%      v = loadvec('*.vc7');
%      histscal_disp(vec2scal(v,'rot'));
%
%   See also HISTF, HISTVEC_DISP.


%   F. Moisy, moisy_at_fast.u-psud.fr
%   Revision: 1.05,  Date: 2008/10/20
%   This function is part of the PIVMat Toolbox


% History:
% 2004/04/26: v1.00, first version.
% 2004/09/23: v1.01, the mean of the Gaussian histo is not subtracted.
% 2005/02/23: v1.02, cosmetics
% 2005/09/06: v1.03, id.
% 2006/10/06: v1.04, do not create a new figure at each plot
% 2008/10/20: v1.05, now works with non-double scalar images (IMX/IM7)


error(nargchk(1,4,nargin));

maxframe=length(s);

if nargin<2, smooth=0; end;
if smooth==0, smooth=maxframe; end;

if isfield(s,'vx')
    s=vec2scal(s,'ux');
end

for i=1:numel(s)   % new v1.05
    s(i).w = double(s(i).w);
end

if nargin<3, bin=0; end;
% if bin not defined, computes if from the statistics of the 1st frame
if bin==0,
    stat=statf(s(1));
    maxbin=20*stat.rms;
    bin=linspace(-maxbin, maxbin, 200);
end;

if nargin<4, opt='ngl'; end;

% average the histograms:
for numfield=1:floor(maxframe/smooth),
    range=((numfield-1)*smooth+1):(numfield*smooth);
    sm_hi(numfield,:)=histf(s(range),bin);
    sm_stat(numfield)=statf(s(range));
    % normalise les histos
    if findstr(lower(opt),'n'),
        sm_hi(numfield,:)=sm_hi(numfield,:)/(sum(sm_hi(numfield,:),2)*(bin(2)-bin(1)));
    end;
end;


% displays the histograms
for numfield=1:floor(maxframe/smooth)    
    if findstr(lower(opt),'l')
        semilogy(bin,sm_hi(numfield,:),'ro');
        maxy=10^(ceil(log10(max(sm_hi(numfield,:)))));
        miny=10^(floor(log10(min(sm_hi(numfield,find(sm_hi(numfield,:)~=0))))));        
    else
        plot(bin,sm_hi(numfield,:),'ro');
        maxy=1.2*max(sm_hi(numfield,:));
        miny=0;
    end;
    axis([-15*sm_stat(numfield).std 15*sm_stat(numfield).std miny maxy]);
    
    line([0 0],[miny maxy]);  
    xlabel([s(1).namew ' (' s(1).unitw ')']);
    if findstr(lower(opt),'n'), ylabel('pdf');
    else ylabel('Histogram'); end;
    
    if maxframe==1
        title(texliteral([s(1).setname ' / ' s(1).name]));
    else
        title(texliteral([s(1).setname ' / ' s(1).name '..' s(end).name]));
    end;
    
    annotation('textbox',[.15 .80 .15 .1],'String',...
        ['Mean = ' num2str(sm_stat(numfield).mean) ' \pm ' num2str(sm_stat(numfield).std) ' ' s(1).unitw]);
    
    if findstr(lower(opt),'g'),
        hold on;
        bin_gauss=linspace(bin(1), bin(length(bin)),2000);
%        gauss_vx=1/(sqrt(2*pi)*sm_stat(numfield).std)*exp(-0.5*(bin_gauss - sm_stat(numfield).mean).^2 / sm_stat(numfield).std^2);
        gauss_vx=1/(sqrt(2*pi)*sm_stat(numfield).std)*exp(-0.5*(bin_gauss).^2 / sm_stat(numfield).std^2);
        if findstr(lower(opt),'l'),
            semilogy(bin_gauss,gauss_vx,'b-');   
        else
            plot(bin_gauss,gauss_vx,'b-');   
        end;
        hold off;
    end;
    if ~(smooth==maxframe),
        pause;
    end;
end;
