function histvec_disp(v,smooth,bin,opt)
%HISTVEC_DISP  Displays distributions of vector fields.
%   HISTVEC_DISP(V) displays the distributions of the components of the
%   vector field(s) V.
%
%   HISTVEC_DISP(V,SMOOTH) computes the velocity distributions averaged
%   over SMOOTH consecutive fields. Use SMOOTH=1 for no smoothing, or
%   SMOOTH=0 to smooth over all the fields (by default).
%
%   HISTVEC_DISP(V,SMOOTH,BIN) computes the histograms among the bins
%   specified by the vector BIN. If this argument is not present, the
%   appropriate range of bins is estimated from the rms of the first
%   field.
%
%   HISTVEC_DISP(...,OPT), where OPT is a string that may contain one or
%   more of the following letters (all by default):
%        'n': normalize the histogram (pdf)
%        'g': also displays a gaussian fit
%        'l': log-coordinate for the y axis
%
%   Example:
%      v = loadvec('*.vc7');
%      histvec_disp(v,5,linspace(-0.5,0.5,50),'ngl');
%
%   See also HISTF, STATF, HISTSCAL_DISP.


%   F. Moisy, moisy_at_fast.u-psud.fr
%   Revision: 1.04,  Date: 2011/03/25
%   This function is part of the PIVMat Toolbox


% History:hi
% 2004/04/26: v1.00, first version.
% 2005/02/23: v1.01, cosmetics
% 2005/09/06: v1.02, id.
% 2006/04/28: v1.03, now accepts filenames as input argument
% 2011/03/25: v1.04, loadset removed

error(nargchk(1,4,nargin));

if (ischar(v) || iscellstr(v)), v=loadvec(v); end % new v1.03
maxframe=length(v);

if nargin<2, smooth=0; end
if smooth==0, smooth=maxframe; end

if nargin<3, bin=0; end
% if bin not defined, computes it from the statistics of the 1st frame
if bin==0,
    stat_vx = statf(v(1));
    maxbin=10*stat_vx.rms;
    bin=linspace(-maxbin, maxbin, 100);
end

if nargin<4, opt='ngl'; end

% average the histograms:
for numfield=1:floor(maxframe/smooth),
    range=((numfield-1)*smooth+1):(numfield*smooth);
    sm_hix(numfield,:)=histf(vec2scal(v(range),'ux'),bin);
    sm_hiy(numfield,:)=histf(vec2scal(v(range),'uy'),bin);
    [sm_stat_vx(numfield),sm_stat_vy(numfield)]=statf(v(range));
    % normalise the histograms
    if findstr(lower(opt),'n'),
        sm_hix(numfield,:)=sm_hix(numfield,:)/(sum(sm_hix(numfield,:),2)*(bin(2)-bin(1)));
        sm_hiy(numfield,:)=sm_hiy(numfield,:)/(sum(sm_hiy(numfield,:),2)*(bin(2)-bin(1)));
    end
end


% displays the histograms
h=figure;
for numfield=1:floor(maxframe/smooth),
    range=((numfield-1)*smooth+1):(numfield*smooth);
    windowname=[v(1).setname ', ' num2str(range(1)) '..' num2str(range(length(range)))];
    set(h,'Name',windowname);
    
    if findstr(lower(opt),'l'),
        semilogy(bin,sm_hix(numfield,:),'ro',bin,sm_hiy(numfield,:),'bs');
        maxy=10^(ceil(log10(max([sm_hix(numfield,:) sm_hiy(numfield,:)]))));
        miny=10^(floor(log10(min([sm_hix(numfield,find(sm_hix(numfield,:)~=0)) sm_hiy(numfield,find(sm_hiy(numfield,:)~=0))]))));        
    else
        plot(bin,sm_hix(numfield,:),'ro',bin,sm_hiy(numfield,:),'bs');
        maxy=1.2*max([sm_hix(numfield,:) sm_hiy(numfield,:)]);
        miny=0;
    end
    axis([-6*sm_stat_vy(numfield).std 6*sm_stat_vy(numfield).std miny maxy]);
    
    line([0 0],[miny maxy]);  
    xlabel('V_x(circle),  V_y(square) (m/s)');
    if findstr(lower(opt),'n'), ylabel('pdf');
    else ylabel('Histogram'); end
    
    if maxframe==1
        title(['Distribution of velocity, Mean =(' num2str(sm_stat_vx(numfield).mean) ', ' num2str(sm_stat_vy(numfield).mean) ') m/s']);
    else
        title(['Distribution of velocity, frames ' num2str(range(1)) '..' num2str(range(length(range))) ',  Mean =(' num2str(sm_stat_vx(numfield).mean) ', ' num2str(sm_stat_vy(numfield).mean) ') m/s']);
    end

    if findstr(lower(opt),'g'),
        hold on;
        bin_gauss=linspace(bin(1), bin(length(bin)),2000);
        gauss_vx=max(sm_hix(numfield,:))*exp(-0.5*(bin_gauss - sm_stat_vx(numfield).mean).^2 / sm_stat_vx(numfield).std^2);
        gauss_vy=max(sm_hiy(numfield,:))*exp(-0.5*(bin_gauss - sm_stat_vy(numfield).mean).^2 / sm_stat_vy(numfield).std^2);    
        if findstr(lower(opt),'l'),
            semilogy(bin_gauss,gauss_vx,'r-',bin_gauss,gauss_vy,'b-');   
        else
            plot(bin_gauss,gauss_vx,'r-',bin_gauss,gauss_vy,'b-');   
        end
        hold off;
    end
    if ~(smooth==maxframe),
        pause;
    end
end
