function statvec_disp(v,smooth)
%STATVEC_DISP  Displays statistics of vector fields.
%   STATVEC_DISP(V) displays statistics of a set of vector fields as a
%   function of the field number.
%
%   The first 3 subplots show the evolution of the velocity statistics,
%   for each velocity component:
%    - Mean velocity <Vx> and <Vy>  (where <> is the spatial average)
%    - rms velocity <Vx^2>^(1/2) and <Vy^2>^(1/2)
%    - standard deviation <(Vx-<Vx>)^2>^(1/2) and <(Vy-<Vy>)^2>^(1/2)
%
%   The last subplot shows the energy (summed over the two velocity
%   components):
%     E_total = (<Vx^2> + <Vy^2>) / 2
%     E_turbulent = (<(Vx-<Vx>)^2> + <(Vy-<Vy>)^2>) / 2
%
%   STATVEC_DISP(V,SMOOTH) smoothes the statistics over SMOOTH consecutive
%   vector fields. Use SMOOTH=1 for no smoothing (by default).  Use
%   SMOOTH=0 for smoothing over the whole fields (display only one point).
%
%   Example:
%      v = loadvec('*.vc7');
%      statvec_disp(v,10);
%
%   See also STATF, HISTF, HISTVEC_DISP.


%   F. Moisy, moisy_at_fast.u-psud.fr
%   Revision: 1.06,  Date: 2012/05/31
%   This function is part of the PIVMat Toolbox


% History:
% 2004/04/22: v1.00, first version.
% 2005/02/23: v1.01, cosmetics 
% 2005/09/06: v1.02, id.
% 2005/09/15: v1.03, output arg suppressed.
% 2006/04/27: v1.04, optimized; now reads the setname of v
% 2011/03/25: v1.05, loadset removed
% 2012/05/31: v1.06, help text improved

error(nargchk(1,2,nargin));

if (ischar(v) || iscellstr(v)), v=loadvec(v); end % new v1.04
maxframe=length(v);

if nargin<2, smooth=1; end
if smooth==0, smooth=maxframe; end

% average the statistics
sm_frame=1:floor(maxframe/smooth);
for numfield=sm_frame
    range=((numfield-1)*smooth+1):(numfield*smooth);
    sm_stat_vx(numfield)=statf(vec2scal(v(range),'ux'));
    sm_stat_vy(numfield)=statf(vec2scal(v(range),'uy'));
end

% displays the results
windowname=[v(1).setname ', 1..' num2str(maxframe)];
h=figure;
set(h,'Name',windowname);

subplot(2,2,1);
plot(sm_frame*smooth,[sm_stat_vx(sm_frame).mean],'ro-', sm_frame*smooth, [sm_stat_vy(sm_frame).mean],'bs-');
xlabel('field num');
ylabel('m/s');
legend('<V_x>','<V_y>');
title('Mean velocity');
line([min(sm_frame*smooth) max(sm_frame*smooth)],[0 0])
max_mean=max(max(abs([sm_stat_vx(:).mean]),abs([sm_stat_vy(:).mean])));
if ~(smooth==maxframe), axis([min(sm_frame*smooth) max(sm_frame*smooth) -2*max_mean 2*max_mean]); end;

subplot(2,2,2);
ke_tot=([sm_stat_vx(sm_frame).rms].^2 + [sm_stat_vy(sm_frame).rms].^2)/2;
ke_turb=([sm_stat_vx(sm_frame).std].^2 + [sm_stat_vy(sm_frame).std].^2)/2;
plot(sm_frame*smooth,ke_tot,'ro-', sm_frame*smooth,ke_turb,'bs-');
legend('E total','E turb');
xlabel('field num');
ylabel('E (m/s)^2');
title('2D Total & Turbulent kinetic energy');
line([min(sm_frame) max(sm_frame)],[0 0])
if ~(smooth==maxframe), axis([min(sm_frame*smooth) max(sm_frame*smooth) 0 1.2*max(ke_tot)]); end;

subplot(2,2,3);
plot(sm_frame*smooth,[sm_stat_vx(sm_frame).std],'ro-', sm_frame*smooth, [sm_stat_vy(sm_frame).std],'bs-');
xlabel('field num');
ylabel('m/s');
legend('std(V_x)','std(V_y)');
title('Standard deviation');
max_std=max(max([sm_stat_vx(:).std], [sm_stat_vy(:).std]));
if ~(smooth==maxframe), axis([min(sm_frame*smooth) max(sm_frame*smooth) 0 1.2*max_std]); end;

subplot(2,2,4);
plot(sm_frame*smooth,[sm_stat_vx(sm_frame).rms],'ro-', sm_frame*smooth, [sm_stat_vy(sm_frame).rms],'bs-');
xlabel('field num');
ylabel('m/s');
legend('rms(V_x)','rms(V_y)');
title('rms velocity');
max_rms=max(max([sm_stat_vx(:).rms], [sm_stat_vy(:).rms]));
if ~(smooth==maxframe),  axis([min(sm_frame*smooth) max(sm_frame*smooth) 0 1.2*max_rms]); end;
