function vv=subaverf(v,opt)
%SUBAVERF  Subtract the spatial or ensemble average of a field
%   FF = SUBAVERF(F) subtracts the spatial average from the vector/scalar
%   field F. If F is an array of fields, SUBAVERF(F) subtracts the
%   individual spatial average from each field.
%
%   FF = SUBAVERF(F,AXIS) subtracts the spatial average (computed by
%   SPAVERF), averaged over the direction AXIS, where AXIS='X', 'Y' or
%   'XY'. Default is 'XY'.
%
%   FF = SUBAVERF(F,'e') subtracts from each F the field averaged over all
%   F, as computed from AVERF. The field subtracted is an "ensemble" or
%   "phase" or "temporal" average.
%
%   If no output argument, the result is displayed by SHOWF.
%
%   Example:
%     v = loadvec('*.VC7');
%     showf(subaverf(v,'e'));
%
%   Note: To subtract both an ensemble average and a spatial average, use
%   FF = OPERF('-', F, AVERF(SPAVERF(F,'...')));
%
%   See also AVERF, SPAVERF, AZAVERF.


%   F. Moisy, moisy_at_fast.u-psud.fr
%   Revision: 1.11,  Date: 2009/11/24
%   This function is part of the PIVMat Toolbox


% History:
% 2005/02/16: v1.00, first version (originally called 'submeanvec')
% 2005/02/18: v1.01, bug fixed for nx <> ny
% 2005/09/07: v1.02, history and arg check added.
% 2005/10/06: v1.03, option 'ensemble' added. now called 'subavervec'.
% 2005/10/11: v1.10, subaverf (from subavervec 1.03, 2005/10/06)
% 2009/11/24: v1.11, cosmetics

error(nargchk(1,2,nargin));

if (ischar(v) || iscellstr(v))
    v=loadvec(v);
end 

if nargin<2
    opt='xy';
end

vv=v;

vecmode=isfield(v(1),'vx');   % 1 for vector field, 0 for scalar field

if findstr(lower(opt),'e'),
    ev=averf(v);
end

for i=1:length(v)
    if findstr(lower(opt),'e')   %new v1.03
        if vecmode
            vv(i).vx = (v(i).vx - ev.vx).*logical(v(i).vx);  % modified v1.11
            vv(i).vy = (v(i).vy - ev.vy).*logical(v(i).vy);
        else
            vv(i).w = (v(i).w - ev.w).*logical(v(i).w);
        end
    else
        av=spaverf(v(i),opt);
        if vecmode
            vv(i).vx = (v(i).vx - av.vx).*logical(v(i).vx);
            vv(i).vy = (v(i).vy - av.vy).*logical(v(i).vy);
        else
            vv(i).w = (v(i).w - av.w).*logical(v(i).w);
        end
    end
    
    vv(i).history = {v(i).history{:} ['subaverf(ans, ''' opt ''')']}';
end

if nargout==0
    showf(vv);
    clear vv
end
